@@ -82,6 +82,7 @@ typedef enum
   ZEITGEIST_EVENT_INTERPRETATION,
   ZEITGEIST_EVENT_MANIFESTATION,
   ZEITGEIST_EVENT_ACTOR,
+  ZEITGEIST_EVENT_ORIGIN,
 } ZeitgeistEventDataOffset;
 
 typedef struct
@@ -91,6 +92,7 @@ typedef struct
   gchar      *interpretation;
   gchar      *manifestation;
   gchar      *actor;
+  gchar      *origin;
   GPtrArray  *subjects;
   GByteArray *payload;
 } ZeitgeistEventPrivate;
@@ -430,6 +432,57 @@ zeitgeist_event_add_subject (ZeitgeistEvent   *event,
   g_object_ref_sink (subject);
 }
 
+/**
+ * zeitgeist_event_get_origin:
+ * @event: The event to get the origin from
+ *
+ * Get the origin of an event.
+ * This differs from a subject's origin, as it describes where the event comes
+ * from, not where it resides.
+ *
+ * Returns: The origin of @event.
+ *
+ * Since: 0.3.14
+ */
+const gchar*
+zeitgeist_event_get_origin (ZeitgeistEvent *event)
+{
+  g_return_val_if_fail (ZEITGEIST_IS_EVENT (event), NULL);
+
+  ZeitgeistEventPrivate* priv = ZEITGEIST_EVENT_GET_PRIVATE (event);
+  
+  return priv->origin;
+}
+
+/**
+ * zeitgeist_event_set_origin:
+ * @event: The event to set the origin of
+ * @origin: The origin to set
+ *
+ * Set the origin of an event.
+ * This differs from a subject's origin, as it describes where the event comes
+ * from, not where it resides.
+ *
+ * Since: 0.3.14
+ */
+void
+zeitgeist_event_set_origin (ZeitgeistEvent *event,
+                            const gchar    *origin)
+{
+  g_return_if_fail (ZEITGEIST_IS_EVENT (event));
+
+  ZeitgeistEventPrivate* priv = ZEITGEIST_EVENT_GET_PRIVATE (event);
+
+  gchar* copy = g_strdup (origin);
+
+  if (priv->origin)
+    {
+      g_free (priv->origin);
+    }
+  
+  priv->origin = copy;
+}
+
 /**
  * zeitgeist_event_get_payload:
  * @event: The event to get the payload for
@@ -439,7 +492,6 @@ zeitgeist_event_add_subject (ZeitgeistEvent   *event,
  * Returns: The event payload or %NULL if unset. Do not free. If you want to
  *          keep the subject around you need to g_byte_array_ref() it.
  */
-
 GByteArray*
 zeitgeist_event_get_payload (ZeitgeistEvent *event)
 {
@@ -491,6 +543,7 @@ zeitgeist_event_init (ZeitgeistEvent *object)
   priv->interpretation = NULL;
   priv->manifestation = NULL;
   priv->actor = NULL;
+  priv->origin = NULL;
   priv->subjects = g_ptr_array_new_with_free_func (
                                                 (GDestroyNotify)g_object_unref);
   priv->payload = NULL;
@@ -506,14 +559,15 @@ zeitgeist_event_finalize (GObject *object)
 
   if (priv->subjects)
     {
-      /* Subjects are unreffed by the free-func of the GPtrArray  */
+      /* Subjects are unreffed by the free-func of the GPtrArray */
       g_ptr_array_unref (priv->subjects);
       priv->subjects = NULL;
     }
   
   zeitgeist_event_set_interpretation (event, NULL);
   zeitgeist_event_set_manifestation (event, NULL);
-  zeitgeist_event_set_actor (event, NULL);  
+  zeitgeist_event_set_actor (event, NULL);
+  zeitgeist_event_set_origin (event, NULL);
   zeitgeist_event_set_payload (event, NULL);
 
   G_OBJECT_CLASS (zeitgeist_event_parent_class)->finalize (object);
@@ -550,6 +604,7 @@ zeitgeist_event_new (void)
   return self;
 }
 
+// FIXME: add `origin' parameter
 /** 
  * zeitgeist_event_new_full:
  * @interpretation: The interpretation type of the event.
@@ -691,6 +746,9 @@ zeitgeist_event_new_from_variant (GVariant *event)
   zeitgeist_event_set_actor (result,  str[0] == '\0' ? NULL : str);
   g_free (str);
 
+  if (g_variant_iter_loop (event_data, "s", &str))
+    zeitgeist_event_set_origin (result, str[0] == '\0' ? NULL : str);
+
   /* Build the list of subjects */
   while (g_variant_iter_loop (subjects, "as", &subject_data))
   {
@@ -732,6 +790,9 @@ zeitgeist_event_new_from_variant (GVariant *event)
     zeitgeist_subject_set_storage (subject, str[0] == '\0' ? NULL : str);
     g_free (str);
 
+    if (g_variant_iter_loop (subject_data, "s", &str))
+      zeitgeist_subject_set_current_uri (subject, str[0] == '\0' ? NULL : str);
+
     zeitgeist_event_add_subject (result, subject);
   }
 
@@ -799,6 +860,7 @@ zeitgeist_event_to_variant (ZeitgeistEvent *event)
   g_variant_builder_add (&b, "s", (bif = zeitgeist_event_get_interpretation(event), bif ? bif : ""));
   g_variant_builder_add (&b, "s", (bif = zeitgeist_event_get_manifestation(event), bif ? bif : ""));
   g_variant_builder_add (&b, "s", (bif = zeitgeist_event_get_actor (event), bif ? bif : ""));
+  g_variant_builder_add (&b, "s", (bif = zeitgeist_event_get_origin (event), bif ? bif : ""));
   g_variant_builder_close (&b);
 
   /* Build subjects */
@@ -815,6 +877,7 @@ zeitgeist_event_to_variant (ZeitgeistEvent *event)
       g_variant_builder_add (&b, "s", (bif = zeitgeist_subject_get_mimetype(su), bif ? bif : ""));
       g_variant_builder_add (&b, "s", (bif = zeitgeist_subject_get_text(su), bif ? bif : ""));
       g_variant_builder_add (&b, "s", (bif = zeitgeist_subject_get_storage(su), bif ? bif : ""));
+      g_variant_builder_add (&b, "s", (bif = zeitgeist_subject_get_current_uri(su), bif ? bif : ""));
       g_variant_builder_close (&b);
     }
   g_variant_builder_close (&b);
